@@ -119,8 +119,8 @@ if __name__ == '__main__':
     parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
     parser.add_argument('-port', action='store', default='1433', help='target MSSQL port (default 1433)')
     parser.add_argument('-db', action='store', help='MSSQL database instance (default None)')
-    parser.add_argument('-windows-auth', action='store_true', default = 'False', help='whether or not to use Windows '
-                                                                                      'Authentication (default False)')
+    parser.add_argument('-windows-auth', action='store_true', default=False, help='whether or not to use Windows '
+                                                                                  'Authentication (default False)')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('-file', type=argparse.FileType('r'), help='input file with commands to execute in the SQL shell')
 
