@@ -1,57 +1,45 @@
-# Copyright (c) 2003-2014 CORE Security Technologies
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: scmr.py 1244 2014-06-26 13:54:47Z bethus@gmail.com $
-#
-# Author: Alberto Solino
+# Author: Alberto Solino (@agsolino)
 #
 # Description:
 #   [MS-SCMR] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://code.google.com/p/impacket/source/browse/#svn%2Ftrunk%2Fimpacket%2Ftestcases%2FSMB_RPC
+#   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file. 
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
-from struct import unpack, pack
+
 from impacket import system_errors
+from impacket.dcerpc.v5.dtypes import NULL, DWORD, LPWSTR, ULONG, BOOL, LPBYTE, ULONGLONG, PGUID, USHORT, LPDWORD, WSTR, \
+    GUID, PBOOL, WIDESTR
+from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRPOINTER, NDRPOINTERNULL, NDRUniConformantArray, NDRUNION
+from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket.uuid import uuidtup_to_bin
-from impacket.dcerpc.v5 import ndr
-from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRPOINTER, NDRPOINTERNULL, NDRUniConformantArray, PNDRUniConformantArray, NDRBOOLEAN, NDRUniFixedArray, NDRUNION, NULL
-from impacket.dcerpc.v5.dtypes import *
 
 MSRPC_UUID_SCMR = uuidtup_to_bin(('367ABB81-9844-35F1-AD32-98F038001003', '2.0'))
 
-class DCERPCSessionError(Exception):
-    def __init__( self, packet = None, error_code = None):
-        Exception.__init__(self)
-        self.packet = packet
-        if packet is not None:
-            self.error_code = packet['ErrorCode']
-        else:
-            self.error_code = error_code
-       
-    def get_error_code( self ):
-        return self.error_code
- 
-    def get_packet( self ):
-        return self.packet
+class DCERPCSessionError(DCERPCException):
+    def __init__(self, error_string=None, error_code=None, packet=None):
+        DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
         key = self.error_code
-        if (system_errors.ERROR_MESSAGES.has_key(key)):
+        if key in system_errors.ERROR_MESSAGES:
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'SCMR SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
         else:
-            return 'SCMR SessionError: unknown error code: 0x%x' % (self.error_code)
+            return 'SCMR SessionError: unknown error code: 0x%x' % self.error_code
 
 ################################################################################
 # CONSTANTS
@@ -216,11 +204,15 @@ SERVICE_TRIGGER_DATA_TYPE_STRING = 0x00000002
 # STRUCTURES
 ################################################################################
 
-class SC_RPC_HANDLE(NDR):
-    align = 1
+class BYTE_ARRAY(NDRUniConformantArray):
+    item = 'c'
+
+class SC_RPC_HANDLE(NDRSTRUCT):
     structure =  (
         ('Data','20s=""'),
     )
+    def getAlignment(self):
+        return 1
 
 SC_NOTIFY_RPC_HANDLE = SC_RPC_HANDLE
 
@@ -677,7 +669,7 @@ class RQueryServiceObjectSecurity(NDRCALL):
 
 class RQueryServiceObjectSecurityResponse(NDRCALL):
     structure = (
-        ('lpSecurityDescriptor',LPBYTE),
+        ('lpSecurityDescriptor', BYTE_ARRAY),
         ('pcbBytesNeeded',BOUNDED_DWORD_256K),
         ('ErrorCode', DWORD),
     )
@@ -1183,15 +1175,25 @@ def hRLockServiceDatabase(dce, hSCManager):
     request['hSCManager'] = hSCManager
     return dce.request(request)
 
-def hRQueryServiceObjectSecurity(dce, hService, dwSecurityInformation, cbBufSize ):
+
+def hRQueryServiceObjectSecurity(dce, hService, dwSecurityInformation, cbBufSize=0):
     request = RQueryServiceObjectSecurity()
     request['hService'] = hService
     request['dwSecurityInformation'] = dwSecurityInformation
     request['cbBufSize'] = cbBufSize
-    return dce.request(request)
+    try:
+        resp = dce.request(request)
+    except DCERPCSessionError as e:
+        if e.get_error_code() == system_errors.ERROR_INSUFFICIENT_BUFFER:
+            resp = e.get_packet()
+            request['cbBufSize'] = resp['pcbBytesNeeded']
+            resp = dce.request(request)
+        else:
+            raise
+    return resp
 
 def hRSetServiceObjectSecurity(dce, hService, dwSecurityInformation, lpSecurityDescriptor, cbBufSize ):
-    request = RSetServiceObjectSecurityCall()
+    request = RSetServiceObjectSecurity()
     request['hService'] = hService
     request['dwSecurityInformation'] = dwSecurityInformation
     request['cbBufSize'] = cbBufSize
@@ -1292,7 +1294,7 @@ def hREnumServicesStatusW(dce, hSCManager, dwServiceType=SERVICE_WIN32_OWN_PROCE
 
     try:
         resp = dce.request(enumServicesStatus)
-    except DCERPCSessionError, e:
+    except DCERPCSessionError as e:
         if e.get_error_code() == system_errors.ERROR_MORE_DATA:
             resp = e.get_packet()
             enumServicesStatus['cbBufSize'] = resp['pcbBytesNeeded']
@@ -1301,19 +1303,22 @@ def hREnumServicesStatusW(dce, hSCManager, dwServiceType=SERVICE_WIN32_OWN_PROCE
             raise
     
     # Now we're supposed to have all services returned. Now we gotta parse them
+
     enumArray = NDRUniConformantArray()
     enumArray.item = ENUM_SERVICE_STATUSW2
-    data = ''.join(resp['lpBuffer'])
-    data = pack('<L', resp['lpServicesReturned']) + data
+
+    enumArray.setArraySize(resp['lpServicesReturned'])
+
+    data = b''.join(resp['lpBuffer'])
     enumArray.fromString(data)
     data = data[4:]
     # Since the pointers here are pointing to the actual data, we have to reparse
     # the referents
     for record in enumArray['Data']:
-        offset =  record.fields['lpDisplayName'].fields['ReferentID']
+        offset =  record.fields['lpDisplayName'].fields['ReferentID']-4
         name = WIDESTR(data[offset:])
         record['lpDisplayName'] = name['Data']
-        offset =  record.fields['lpServiceName'].fields['ReferentID']
+        offset =  record.fields['lpServiceName'].fields['ReferentID']-4
         name = WIDESTR(data[offset:])
         record['lpServiceName'] = name['Data']
 
@@ -1339,7 +1344,7 @@ def hRQueryServiceConfigW(dce, hService):
     queryService['cbBufSize'] = 0
     try:
         resp = dce.request(queryService)
-    except DCERPCSessionError, e:
+    except DCERPCSessionError as e:
         if e.get_error_code() == system_errors.ERROR_INSUFFICIENT_BUFFER:
             resp = e.get_packet()
             queryService['cbBufSize'] = resp['pcbBytesNeeded']
@@ -1362,7 +1367,6 @@ def hRStartServiceW(dce, hService, argc=0, argv=NULL ):
     if argc == 0:
         startService['argv'] = NULL
     else:
-        items = []
         for item in argv:
             itemn = LPWSTR()
             itemn['Data'] = checkNullString(item)
@@ -1392,4 +1396,3 @@ def hREnumServiceGroupW(dce, hSCManager, dwServiceType, dwServiceState, cbBufSiz
     enumServiceGroup['lpResumeIndex'] = lpResumeIndex
     enumServiceGroup['pszGroupName'] = pszGroupName
     return dce.request(enumServiceGroup)
-
