@@ -23,6 +23,7 @@ class NTLMRelayxConfig:
         self.listeningPort = None
 
         self.domainIp = None
+
         self.machineAccount = None
         self.machineHashes = None
         self.target = None
@@ -36,6 +37,8 @@ class NTLMRelayxConfig:
         self.ipv6 = False
         self.remove_mic = False
 
+        self.command = None
+
         # WPAD options
         self.serve_wpad = False
         self.wpad_host = None
@@ -50,9 +53,17 @@ class NTLMRelayxConfig:
 
         # SMB options
         self.exeFile = None
-        self.command = None
         self.interactive = False
         self.enumLocalAdmins = False
+        self.SMBServerChallenge = None
+
+        # RPC options
+        self.rpc_mode = None
+        self.rpc_use_smb = False
+        self.auth_smb = ''
+        self.smblmhash = None
+        self.smbnthash = None
+        self.port_smb = 445
 
         # LDAP options
         self.dumpdomain = True
@@ -77,6 +88,9 @@ class NTLMRelayxConfig:
         # WebDAV options
         self.serve_image = False
 
+    def setSMBChallenge(self, value):
+        self.SMBServerChallenge = value
+
     def setSMB2Support(self, value):
         self.smb2support = value
 
@@ -136,7 +150,7 @@ class NTLMRelayxConfig:
     def setRandomTargets(self, randomtargets):
         self.randomtargets = randomtargets
 
-    def setLDAPOptions(self, dumpdomain, addda, aclattack, validateprivs, escalateuser, addcomputer, delegateaccess):
+    def setLDAPOptions(self, dumpdomain, addda, aclattack, validateprivs, escalateuser, addcomputer, delegateaccess, dumplaps, dumpgmsa, sid):
         self.dumpdomain = dumpdomain
         self.addda = addda
         self.aclattack = aclattack
@@ -144,10 +158,29 @@ class NTLMRelayxConfig:
         self.escalateuser = escalateuser
         self.addcomputer = addcomputer
         self.delegateaccess = delegateaccess
+        self.dumplaps = dumplaps
+        self.dumpgmsa = dumpgmsa
+        self.sid = sid
 
     def setMSSQLOptions(self, queries):
         self.queries = queries
 
+    def setRPCOptions(self, rpc_mode, rpc_use_smb, auth_smb, hashes_smb, rpc_smb_port):
+        self.rpc_mode = rpc_mode
+        self.rpc_use_smb = rpc_use_smb
+
+        import re
+        auth_re = re.compile('(?:(?:([^/:]*)/)?([^:]*)(?::(.*))?)?')
+        self.smbdomain, self.smbuser, self.smbpass =  auth_re.match(auth_smb).groups('')
+
+        if hashes_smb is not None:
+            self.smblmhash, self.smbnthash = hashes_smb.split(':')
+        else:
+            self.smblmhash = ''
+            self.smbnthash = ''
+
+        self.rpc_smb_port = rpc_smb_port
+
     def setInteractive(self, interactive):
         self.interactive = interactive
 
