@@ -242,6 +242,8 @@ class Structure:
 
         # asciiz specifier
         if format[:1] == 'z':
+            if isinstance(data,bytes):
+                return data + b('\0')
             return bytes(b(data)+b('\0'))
 
         # unicode specifier
@@ -635,3 +637,22 @@ def hexdump(data, indent = ''):
         line += ''.join(pretty_print(x) for x in x[i:i+16] )
         print (line)
         i += 16
+
+def parse_bitmask(dict, value):
+    ret = ''
+    
+    for i in range(0, 31):
+        flag = 1 << i
+
+        if value & flag == 0:
+            continue
+
+        if flag in dict:
+            ret += '%s | ' % dict[flag]
+        else:
+            ret += "0x%.8X | " % flag
+
+    if len(ret) == 0:
+        return '0'
+    else:
+        return ret[:-3]
