@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2016 CORE Security Technologies
+# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -15,6 +15,7 @@ import array
 import struct
 
 from impacket.helper import ProtocolPacket, Byte, Bit
+from functools import reduce
 
 
 class ArrayBuilder(object):
@@ -55,7 +56,7 @@ class NumBuilder(object):
     def to_ary(self, value0):
         value = value0
         rv = array.array('B')
-        for _ in xrange(self.size):
+        for _ in range(self.size):
             value, mod = divmod(value, 256)
             rv.append(mod)
             
@@ -399,6 +400,5 @@ class SimpleConfig(ProtocolPacket):
     def build_tlv_container(cls):
         return TLVContainer(
             builders=SimpleConfig.BUILDERS, 
-            descs = dict( (v,k) for (k,v) in SCElem.__dict__.iteritems() )
+            descs = dict( (v,k) for (k,v) in SCElem.__dict__.items() )
         )
-    
